import decimal


def half_round(num, format):
    """
    소수 반올림용 함수
    사용법:
    decimal_round(0.1235, '0.1') == 0.1
    decimal_round(0.1235, '0.01') == 0.12
    decimal_round(0.1235, '0.001') == 0.124
    decimal_round(0.1235, '0.0001') == 0.1235
    decimal_round(0.12345, '1') == 0.0
    """
    n = decimal.Decimal(str(num)).quantize(decimal.Decimal(format), rounding=decimal.ROUND_HALF_UP)
    return float(n)


def clear_int(num, clear_with):
    clear_with = cleared_int(intlen(clear_with))

    if intlen(num) == intlen(clear_with):
        raise ValueError('You can not clear same length integer')
    return num // clear_with * clear_with


def intlen(num):
    return len(str(num))


def cleared_int(zero_count):
    return int('1' + ''.zfill(zero_count))



from jackal.settings import jackal_settings
from jackal.structures import JackalBaseStructure, BaseQueryFunction


def structure_loader(key):
    ret_data = dict()
    stru = getattr(jackal_settings, key, None)

    if stru is None:
        stru = jackal_settings.CUSTOM_STRUCTURES.get(key)

        if stru is None:
            return {}

    for cls in stru:
        if issubclass(cls, JackalBaseStructure):
            ret_data.update(cls.get_structure())
        else:
            continue
    return ret_data


def query_function_loader():
    ret_data = dict()
    for cls in jackal_settings.QUERY_FUNCTION_CLASSES:
        if issubclass(cls, BaseQueryFunction):
            ret_data.update(cls.get_function_set())

    return ret_data



from django.db import models
from django.utils import timezone


class SoftDeleteQuerySet(models.QuerySet):
    def delete(self):
        self.update(deleted_at=timezone.now())


class SoftDeleteManager(models.Manager):
    _queryset_class = SoftDeleteQuerySet

    def get_queryset(self):
        kwargs = {'model': self.model, 'using': self._db}
        if hasattr(self, '_hints'):
            kwargs['hints'] = self._hints

        return self._queryset_class(**kwargs).filter(deleted_at__isnull=True)

    @property
    def defaults(self):
        kwargs = {'model': self.model, 'using': self._db}
        if hasattr(self, '_hints'):
            kwargs['hints'] = self._hints
        return self._queryset_class(**kwargs)
